import St from 'gi://St';
import Gio from 'gi://Gio';

import {Extension} from 'resource:///org/gnome/shell/extensions/extension.js';
import * as Main from 'resource:///org/gnome/shell/ui/main.js';

export default class LuminusExtension extends Extension {
    // For modern GNOME. Changes the default scheme.
    _setDefaultColorScheme(scheme) {
        Main.sessionMode.colorScheme = scheme;
        St.Settings.get().notify('color-scheme');
    }

    // For legacy applications.
    //
    // ! Schemes will not switch if dark variation of legacy theme is missing
    //   on target system.
    _setLegacyColorScheme(scheme) {
        const isDark = this._interface.get_string('gtk-theme').endsWith("-dark");

        switch (scheme) {
            case 'default':
            case 'prefer-light':
                if (isDark) {
                    this._interface.set_string(
                        'gtk-theme',
                        this._interface.get_string('gtk-theme').slice(0,-5)
                    );
                }
                break;
            case 'prefer-dark':
                if (!isDark) {
                    this._interface.set_string(
                        'gtk-theme',
                        this._interface.get_string('gtk-theme') + "-dark"
                    );
                }
                break;
            default:
                break;
        }
    }

    enable() {
        // Hook the legacy theme switcher to scheme switch
        this._interface = Gio.Settings.new('org.gnome.desktop.interface');
        this._id = this._interface.connect('changed::color-scheme', () => {
            this._setLegacyColorScheme(
                this._interface.get_string('color-scheme')
            );
        });

        // Change default scheme
        this._savedColorScheme = Main.sessionMode.colorScheme;
        this._setDefaultColorScheme('prefer-light');
    }

    disable() {
        this._setDefaultColorScheme(this._savedColorScheme);

        // Legacy apps scheme should always be reset to light, as it doesn't
        // support the mode switching.
        this._setLegacyColorScheme('default');

        if (this._interface) {
            if (this._id) {
                this._interface.disconnect(this._id);
                this._id = null;
            }
            this._interface = null;
        }
    }
}
